@@ -21,6 +21,9 @@ from dataclasses import field
 from datetime import datetime
 from datetime import timedelta
 from hashlib import md5
+from os.path import exists
+from os.path import isabs
+from os.path import join
 from pathlib import Path
 from subprocess import call
 from subprocess import check_output
@@ -33,6 +36,7 @@ from typing import Optional
 from typing import overload
 from typing import Type
 from typing import Union
+from uuid import uuid4
 
 import pytest
 import testinfra
@@ -332,9 +336,7 @@ class BindMountCreator:
 
         assert self.volume.host_path
         self.volume._vol_name = self.volume.host_path
-        if os.path.isabs(self.volume.host_path) and not os.path.exists(
-            self.volume.host_path
-        ):
+        if isabs(self.volume.host_path) and not exists(self.volume.host_path):
             raise RuntimeError(
                 f"Volume with the host path '{self.volume.host_path}' "
                 "was requested but the directory does not exist"
@@ -392,7 +394,7 @@ def get_volume_creator(
 
 
 _CONTAINER_ENTRYPOINT = "/bin/bash"
-_CONTAINER_STOPSIGNAL = ["--stop-signal", "SIGTERM"]
+_CONTAINER_STOPSIGNAL = ("--stop-signal", "SIGTERM")
 
 
 @enum.unique
@@ -475,6 +477,14 @@ class ContainerBase:
     def __str__(self) -> str:
         return self.url or self.container_id
 
+    @property
+    def _build_tag(self) -> str:
+        """Internal build tag assigned to each immage, either the image url or
+        the container digest prefixed with ``pytest_container:``.
+
+        """
+        return self.url or f"pytest_container:{self.container_id}"
+
     @property
     def local_image(self) -> bool:
         """Returns true if this image has been build locally and has not been
@@ -519,22 +529,36 @@ class ContainerBase:
         )
 
         id_or_url = self.container_id or self.url
-        bash_launch_end = _CONTAINER_STOPSIGNAL + [
-            "-it",
-            id_or_url,
+        container_launch = ("-it", id_or_url)
+        bash_launch_end = (
+            *_CONTAINER_STOPSIGNAL,
+            *container_launch,
             _CONTAINER_ENTRYPOINT,
-        ]
+        )
         if self.entry_point == EntrypointSelection.IMAGE:
-            cmd.extend(["-it", id_or_url])
+            cmd.extend(container_launch)
         elif self.entry_point == EntrypointSelection.BASH:
-            cmd.extend(bash_launch_end)
+            cmd.extend(
+                (
+                    "--entrypoint",
+                    _CONTAINER_ENTRYPOINT,
+                    *_CONTAINER_STOPSIGNAL,
+                    *container_launch,
+                )
+            )
         elif self.entry_point == EntrypointSelection.AUTO:
             if self.custom_entry_point:
-                cmd.extend(["-it", id_or_url, self.custom_entry_point])
+                cmd.extend(
+                    (
+                        "--entrypoint",
+                        self.custom_entry_point,
+                        *container_launch,
+                    )
+                )
             elif container_runtime._get_image_entrypoint_cmd(
                 id_or_url, "Entrypoint"
             ) or container_runtime._get_image_entrypoint_cmd(id_or_url, "Cmd"):
-                cmd.extend(["-it", id_or_url])
+                cmd.extend(container_launch)
             else:
                 cmd.extend(bash_launch_end)
         else:  # pragma: no cover
@@ -587,6 +611,14 @@ class ContainerBaseABC(ABC):
 
         """
 
+    @property
+    @abstractmethod
+    def baseurl(self) -> Optional[str]:
+        """The registry url on which this container is based on, if one
+        exists. Otherwise ``None`` is returned.
+
+        """
+
 
 @dataclass(unsafe_hash=True)
 class Container(ContainerBase, ContainerBaseABC):
@@ -609,6 +641,12 @@ class Container(ContainerBase, ContainerBaseABC):
     def get_base(self) -> "Container":
         return self
 
+    @property
+    def baseurl(self) -> Optional[str]:
+        if self._is_local:
+            return None
+        return self.url
+
 
 @dataclass(unsafe_hash=True)
 class DerivedContainer(ContainerBase, ContainerBaseABC):
@@ -643,6 +681,10 @@ class DerivedContainer(ContainerBase, ContainerBaseABC):
         if not self.base:
             raise ValueError("A base container must be provided")
 
+    @property
+    def baseurl(self) -> Optional[str]:
+        return self.url or self.get_base().baseurl
+
     def __str__(self) -> str:
         return (
             self.container_id
@@ -677,17 +719,18 @@ class DerivedContainer(ContainerBase, ContainerBaseABC):
         # do not build containers without a containerfile and where no build
         # tags are added
         if not self.containerfile and not self.add_build_tags:
-            self.container_id = str(self.get_base())
+            base = self.get_base()
+            self.container_id, self.url = base.container_id, base.url
             return
 
         with tempfile.TemporaryDirectory() as tmpdirname:
-            containerfile_path = os.path.join(tmpdirname, "Dockerfile")
+            containerfile_path = join(tmpdirname, "Dockerfile")
+            iidfile = join(tmpdirname, str(uuid4()))
             with open(containerfile_path, "w") as containerfile:
                 from_id = (
                     self.base
                     if isinstance(self.base, str)
-                    else getattr(self.base, "url", self.base.container_id)
-                    or self.base.container_id
+                    else (getattr(self.base, "url") or self.base._build_tag)
                 )
                 assert from_id
                 containerfile_contents = f"""FROM {from_id}
@@ -747,14 +790,34 @@ class DerivedContainer(ContainerBase, ContainerBaseABC):
                     if self.add_build_tags
                     else []
                 )
-                + ["-f", containerfile_path, str(rootdir)]
+                + [
+                    f"--iidfile={iidfile}",
+                    "-f",
+                    containerfile_path,
+                    str(rootdir),
+                ]
             )
+
             _logger.debug("Building image via: %s", cmd)
-            self.container_id = runtime.get_image_id_from_stdout(
-                check_output(cmd).decode().strip()
+            check_output(cmd)
+
+            self.container_id = runtime.get_image_id_from_iidfile(iidfile)
+
+            assert self._build_tag.startswith("pytest_container:")
+
+            check_output(
+                (
+                    runtime.runner_binary,
+                    "tag",
+                    self.container_id,
+                    self._build_tag,
+                )
             )
+
             _logger.debug(
-                "Successfully build the container image %s", self.container_id
+                "Successfully build the container image %s and tagged it as %s",
+                self.container_id,
+                self._build_tag,
             )
 
 
@@ -774,7 +837,7 @@ class ContainerData:
     #: the testinfra connection to the running container
     connection: Any
     #: the container data class that has been used in this test
-    container: ContainerBase
+    container: Union[Container, DerivedContainer]
     #: any ports that are exposed by this container
     forwarded_ports: List[PortForwarding]
 
@@ -862,6 +925,10 @@ class ContainerLauncher:
 
     _stack: contextlib.ExitStack = field(default_factory=contextlib.ExitStack)
 
+    _cidfile: str = field(
+        default_factory=lambda: join(tempfile.gettempdir(), str(uuid4()))
+    )
+
     def __enter__(self) -> "ContainerLauncher":
         return self
 
@@ -916,7 +983,9 @@ class ContainerLauncher:
         extra_run_args = self.extra_run_args
 
         if self.container_name:
-            extra_run_args.extend(["--name", self.container_name])
+            extra_run_args.extend(("--name", self.container_name))
+
+        extra_run_args.append(f"--cidfile={self._cidfile}")
 
         # We must perform the launches in separate branches, as containers with
         # port forwards must be launched while the lock is being held. Otherwise
@@ -934,14 +1003,17 @@ class ContainerLauncher:
                 )
 
                 _logger.debug("Launching container via: %s", launch_cmd)
-                self._container_id = check_output(launch_cmd).decode().strip()
+                check_output(launch_cmd)
         else:
             launch_cmd = self.container.get_launch_cmd(
                 self.container_runtime, extra_run_args=extra_run_args
             )
 
             _logger.debug("Launching container via: %s", launch_cmd)
-            self._container_id = check_output(launch_cmd).decode().strip()
+            check_output(launch_cmd)
+
+        with open(self._cidfile, "r") as cidfile:
+            self._container_id = cidfile.read(-1).strip()
 
         self._wait_for_container_to_become_healthy()
 
